/*
  Copyright (c) 2012, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#ifndef _FFD_NETIF_H_
#define _FFD_NETIF_H_

#include "types.h"

#include <netinet/in.h>


typedef enum _netif_type_t {
	IF_UNKNOWN = 0,
	IF_WIRED,
	IF_WIRELESS,
	IF_VIRTUAL,
	IF_BRIDGE,
	IF_MESH,
	IF_MAX
} netif_type_t;

typedef void (*netif_cb)(const char *ifname, unsigned ifindex, void *arg);


bool netif_is_mesh(const char *ifname);
char* netif_get_mesh(const char *ifname);
char* netif_get_bridge(const char *ifname);
eth_addr_t netif_get_eth_addr(const char *ifname);
struct in6_addr netif_get_addr(unsigned ifindex, int scope);
void netif_foreach(netif_cb cb, void *arg);

eth_addr_t netif_mesh_get_primary_addr(const char *ifname);

#endif /* _FFD_NETIF_H_ */
